#!/bin/sh
mount -o nosuid,strictatime,mode=755 -t devtmpfs devtmpfs /dev
mount -o nosuid,noexec,nodev -t sysfs sysfs /sys
mount -o nosuid,noexec,nodev -t proc proc /proc

/bin/mkdir /sysroot
/bin/mkdir /opt

ROOTFS=`/sbin/blkid -L rootfs`
if [ x$ROOTFS = "x" ]
then
    ROOTFS=`/sbin/blkid -t PARTLABEL=rootfs -o device`
fi

MODULES=`/sbin/blkid -L modules`
if [ x$MODULES = "x" ]
then
    MODULES=`/sbin/blkid -t PARTLABEL=module -o device`
fi

DATAFS=`/sbin/blkid -L system-data`
if [ x$DATAFS = "x" ]
then
    DATAFS=`/sbin/blkid -t PARTLABEL=system-data -o device`
fi

USERFS=`/sbin/blkid -L user`
if [ x$USERFS = "x" ]
then
    USERFS=`/sbin/blkid -t PARTLABEL=user -o device`
fi

NEED_RESIZEFS=1
USERFS_ENCRYPTED=0
if [ x"$DATAFS" != "x" ]
then
    /sbin/fsck -y $DATAFS
    /bin/mount $DATAFS /opt

    # The device is being encrypted or encrypted.
    if [ -e /opt/etc/.odeprogress -o -e /opt/etc/.ode_* ]
    then
        USERFS_ENCRYPTED=1
    fi

    if [ -e /opt/var/.resizefs_done ]
    then
        NEED_RESIZEFS=0
    else
        echo " " > /opt/var/.resizefs_done
        /bin/umount /opt
    fi
fi

if [ x$ROOTFS != "x" ]
then
    if [ "$NEED_RESIZEFS" = "1" ]
    then
        /sbin/resize2fs -f $ROOTFS
    fi
    /sbin/fsck -y $ROOTFS
    /bin/mount $ROOTFS /sysroot -o rw
else
    echo "WARNING : THERE IS NO ROOTFS."
    exec /bin/sh
fi

if [ x"$DATAFS" != "x" ]
then
    if [ "$NEED_RESIZEFS" = "1" ]
    then
        /sbin/resize2fs -f $DATAFS
        /sbin/fsck -y $DATAFS
        /bin/mount $DATAFS /sysroot/opt
    else
        /bin/mount -M /opt /sysroot/opt
    fi
fi

if [ x"$USERFS" != "x" -a "$USERFS_ENCRYPTED" = "0" ]
then
    if [ "$NEED_RESIZEFS" = "1" ]
    then
        /sbin/resize2fs -f $USERFS
    fi
    /sbin/fsck -y $USERFS
fi

if [ -e /sysroot/opt/sqsh_usr.img ]
then
    if [ ! -d /sysroot/usr ]
    then
        /bin/mkdir /sysroot/usr
    fi
    echo "Mounting Squash FS to /usr..."
    /bin/mount /sysroot/opt/sqsh_usr.img /sysroot/usr
fi

if [ x$MODULES != "x" ]
then
    if [ "$NEED_RESIZEFS" = "1" ]
    then
        /sbin/resize2fs -f $MODULES
    fi
    /sbin/fsck -y $MODULES
    /bin/mount $MODULES /sysroot/usr/lib/modules
fi

cd /sysroot
mkdir -p ./dev ./sys ./proc ./initrd
/sbin/pivot_root . ./initrd
/bin/mount -M ./initrd/dev /dev
/bin/mount -M ./initrd/sys /sys
/bin/mount -M ./initrd/proc /proc
/bin/umount -l ./initrd
/bin/rm -rf ./initrd
/bin/mount -o remount,ro .

INIT=/sbin/init
if [ $$ = 1 ]
then
    exec chroot . $INIT $@
fi
exec /bin/sh
